#pragma once
#include <cstdint>
#include <string>
#include <vector>
#include "extern.h"
#include "misc/delegates.h"

class mixer_track;
class channel_interface;

class CORE_API plugin_host {
    friend class window_manager;
public:
    virtual ~plugin_host();

    virtual bool load_plugin(const char* path) = 0;

    virtual void set_enabled(bool enabled) = 0;
    [[nodiscard]] virtual bool is_enabled() const = 0;

    virtual void on_update_sample_rate(double sample_rate) = 0;
    virtual void on_update_buffer_size(int buffer_size) = 0;
    [[nodiscard]] virtual uint32_t get_input_channels() const = 0;
    [[nodiscard]] virtual uint32_t get_output_channels() const = 0;
    virtual void update_channel_node_name() {}

    virtual void process(uint32_t frame_num) = 0;

    void try_open_editor();
    void try_close_editor();
    virtual void toggle_editor();
    virtual void idle_editor() {}

    [[nodiscard]] virtual bool has_editor() const { return false; }
    virtual void get_editor_size(uint32_t& width, uint32_t& height) const = 0;

    [[nodiscard]] virtual std::string load_name() const { return ""; }
    [[nodiscard]] virtual std::string load_vendor() const { return ""; }

    void init_channel_interface();

    std::string name;
    std::string vendor;
    channel_interface* channel = nullptr;
    std::vector<mixer_track*> owner_tracks;
    bool editor_opened = false;
protected:
    void create_and_open_editor();
    void destroy_editor();
    virtual void open_editor(void* window_handle) = 0;
    virtual void close_editor() = 0;
};
